package main

import (
	"context"
	"fmt"
	"net"
	"os"

	"github.com/jmoiron/sqlx"
	"gitlab.mashplace.com/mashplace/backend/common/api/health"
	"gitlab.mashplace.com/mashplace/backend/common/generic/addr"
	"gitlab.mashplace.com/mashplace/backend/common/generic/grpc"
	logger "gitlab.mashplace.com/mashplace/backend/common/generic/log"
	"gitlab.mashplace.com/mashplace/backend/common/generic/metrics"
	"gitlab.mashplace.com/mashplace/backend/common/generic/tracer"
	notes "gitlab.mashplace.com/mashplace/backend/notes/api"
	"gitlab.mashplace.com/mashplace/backend/notes/internal/model/postgres"
	"gitlab.mashplace.com/mashplace/backend/notes/internal/service"
	"gitlab.mashplace.com/mashplace/backend/notes/version"
)

func main() {
	ctx := context.Background()

	defer tracer.Close()

	conn := fmt.Sprintf(
		"postgres://%s:%s@%s:%s/%s?sslmode=%s",
		os.Getenv("POSTGRES_USERNAME"),
		os.Getenv("POSTGRES_PASSWORD"),
		os.Getenv("POSTGRES_HOST"),
		os.Getenv("POSTGRES_PORT"),
		os.Getenv("POSTGRES_DATABASE"),
		os.Getenv("POSTGRES_SSL_MODE"),
	)

	db := sqlx.MustOpen("postgres", conn)

	repo, err := postgres.NewRepository(db)
	if err != nil {
		panic(err)
	}

	err = repo.Migrate(ctx)
	if err != nil {
		panic(err)
	}

	srv, err := service.NewService(&service.Config{
		Repo: repo,
	})
	if err != nil {
		panic(err)
	}

	listener, err := net.Listen("tcp", addr.Grpc)
	if err != nil {
		panic(err)
	}

	grpcServer, err := grpc.NewServer("notes")
	if err != nil {
		panic(err)
	}

	notes.RegisterNotesServiceServer(grpcServer, srv)

	logger.Infoln("notes started")

	healthServer := version.NewHealthServer()
	health.RegisterHealthServiceServer(grpcServer, healthServer)

	go func() {
		_ = metrics.ConfigureListenAndServe(grpcServer)
	}()

	err = grpcServer.Serve(listener)
	if err != nil {
		logger.Errorln("gRPC error", err)
	}
}
