package postgres

import (
	"context"

	"github.com/jmoiron/sqlx"
	"gitlab.mashplace.com/mashplace/backend/common/api/generic/query"
	"gitlab.mashplace.com/mashplace/backend/common/generic/psqlquery"
	"gitlab.mashplace.com/mashplace/backend/notes/internal/model"
)

type repositoryImplNote struct {
	create        *sqlx.NamedStmt
	serviceImport *sqlx.NamedStmt
	update        *sqlx.NamedStmt
	getByID       *sqlx.NamedStmt
	removeByID    *sqlx.NamedStmt
}

func prepareNote(db *sqlx.DB) (stmt repositoryImplNote, err error) {
	stmt.create, err = prepareNamed(db, queryNoteCreate)
	if err != nil {
		return
	}

	stmt.serviceImport, err = prepareNamed(db, queryNoteServiceImport)
	if err != nil {
		return
	}

	stmt.update, err = prepareNamed(db, queryNoteUpdate)
	if err != nil {
		return
	}

	stmt.getByID, err = prepareNamed(db, queryNoteGetByID)
	if err != nil {
		return
	}

	stmt.removeByID, err = prepareNamed(db, queryNoteRemoveByID)
	if err != nil {
		return
	}

	return
}

const queryNoteCreate = `
insert into notes(
  id,
  correlation_type,
  correlation_id,
  status,
  text,
  userid,
  created,
  updated,
  scheduled
) values (
  default,
  :correlation_type,
  :correlation_id,
  :status,
  :text,
  :userid,
  now(),
  now(),
  :scheduled
) returning
  id,
  correlation_type,
  correlation_id,
  status,
  text,
  userid,
  created,
  updated,
  scheduled
`

func (repo *repositoryImpl) NoteCreate(ctx context.Context, note *model.Note) error {
	return repo.note.create.GetContext(ctx, note, note)
}

const queryNoteServiceImport = `
insert into notes(
  id,
  correlation_type,
  correlation_id,
  status,
  text,
  userid,
  created,
  updated,
  scheduled
) values (
  default,
  :correlation_type,
  :correlation_id,
  :status,
  :text,
  :userid,
  :created,
  :updated,
  :scheduled
) returning
  id,
  correlation_type,
  correlation_id,
  status,
  text,
  userid,
  created,
  updated,
  scheduled
`

func (repo *repositoryImpl) NoteServiceImport(ctx context.Context, note *model.Note) error {
	return repo.note.serviceImport.GetContext(ctx, note, note)
}

const queryNoteUpdate = `
update notes set
  status = :status,
  text = :text,
  scheduled = :scheduled,
  updated = now()
where
  id = :id
returning
  id,
  correlation_type,
  correlation_id,
  status,
  text,
  userid,
  created,
  updated,
  scheduled
`

func (repo *repositoryImpl) NoteUpdate(ctx context.Context, note *model.Note) error {
	return repo.note.update.GetContext(ctx, note, note)
}

const queryNoteGetByID = `
select 
  id,
  correlation_type,
  correlation_id,
  status,
  text,
  userid,
  created,
  updated,
  scheduled
from notes where
  id = :id
`

func (repo *repositoryImpl) NoteGetByID(ctx context.Context, id uint64) (*model.Note, error) {
	n := &model.Note{
		ID: id,
	}

	err := repo.note.getByID.GetContext(ctx, n, n)
	if err != nil {
		return nil, err
	}

	return n, nil
}

const queryNoteRemoveByID = `
delete from notes where
  id = :id
`

func (repo *repositoryImpl) NoteRemoveByID(ctx context.Context, id uint64) error {
	_, err := repo.note.removeByID.ExecContext(ctx, &model.Note{
		ID: id,
	})

	return err
}

func (repo *repositoryImpl) NoteFilterList(
	ctx context.Context,
	filter *query.QueryParams,
) (ms []*model.Note, count uint64, err error) {
	qq, args, err := psqlquery.BuildListQuery(model.Note{}, "notes", filter)
	if err != nil {
		return nil, 0, err
	}

	err = repo.DB.SelectContext(ctx, &ms, qq, args...)
	if err != nil {
		return nil, 0, err
	}

	qq, args, err = psqlquery.BuildCountQuery(model.Note{}, "notes", filter)
	if err != nil {
		return nil, 0, err
	}

	err = repo.DB.GetContext(ctx, &count, qq, args...)
	if err != nil {
		return nil, 0, err
	}

	return
}
