package service

import (
	"context"
	"strconv"

	notes "gitlab.mashplace.com/mashplace/backend/notes/api"
	"gitlab.mashplace.com/mashplace/backend/notes/internal/model"
)

func (srv *serviceImpl) NoteCreate(
	ctx context.Context,
	req *notes.NoteCreateRequest,
) (*notes.NoteCreateResponse, error) {
	status := req.GetStatus()

	if status == notes.Status_STATUS_UNKNOWN {
		status = notes.Status_STATUS_ACTIVE
	}

	m := &model.Note{
		CorrelationType: req.GetCorrelationType(),
		CorrelationID:   req.GetCorrelationId(),
		Status:          status,
		Text:            req.GetText(),
		User:            req.GetUser(),
		Scheduled:       req.GetScheduled().AsTime(),
	}

	err := srv.Repo.NoteCreate(ctx, m)
	if err != nil {
		return nil, err
	}

	return &notes.NoteCreateResponse{
		Note: srv.renderNote(m),
	}, nil
}

func (srv *serviceImpl) NoteUpdate(
	ctx context.Context,
	req *notes.NoteUpdateRequest,
) (*notes.NoteUpdateResponse, error) {
	id, err := strconv.ParseUint(req.GetId(), 10, 64)
	if err != nil {
		return nil, err
	}

	m, err := srv.Repo.NoteGetByID(ctx, id)
	if err != nil {
		return nil, err
	}

	if req.GetStatus() != notes.Status_STATUS_UNKNOWN {
		m.Status = req.GetStatus()
	}

	if req.GetText() != "" {
		m.Text = req.GetText()
	}

	if req.GetScheduled() != nil {
		m.Scheduled = req.GetScheduled().AsTime()
	}

	err = srv.Repo.NoteUpdate(ctx, m)
	if err != nil {
		return nil, err
	}

	return &notes.NoteUpdateResponse{
		Note: srv.renderNote(m),
	}, nil
}

func (srv *serviceImpl) NoteDelete(
	ctx context.Context,
	req *notes.NoteDeleteRequest,
) (*notes.NoteDeleteResponse, error) {
	id, err := strconv.ParseUint(req.GetId(), 10, 64)
	if err != nil {
		return nil, err
	}

	err = srv.Repo.NoteRemoveByID(ctx, id)
	if err != nil {
		return nil, err
	}

	return &notes.NoteDeleteResponse{}, nil
}

func (srv *serviceImpl) NoteFilter(
	ctx context.Context,
	req *notes.NoteFilterRequest,
) (*notes.NoteFilterResponse, error) {
	ms, total, err := srv.Repo.NoteFilterList(ctx, req.GetParams())
	if err != nil {
		return nil, err
	}

	return &notes.NoteFilterResponse{
		Notes: srv.renderNotes(ms),
		Total: int32(total),
	}, nil
}

func (srv *serviceImpl) ServiceNoteImport(
	ctx context.Context,
	req *notes.ServiceNoteImportRequest,
) (*notes.ServiceNoteImportResponse, error) {
	for _, n := range req.GetNotes() {
		status := n.GetStatus()

		if status == notes.Status_STATUS_UNKNOWN {
			status = notes.Status_STATUS_ACTIVE
		}

		m := &model.Note{
			Created:         n.GetCreated().AsTime(),
			Updated:         n.GetUpdated().AsTime(),
			Scheduled:       n.GetScheduled().AsTime(),
			CorrelationID:   n.GetCorrelationId(),
			Text:            n.GetText(),
			User:            n.GetUser(),
			CorrelationType: n.GetCorrelationType(),
			Status:          status,
		}

		err := srv.Repo.NoteServiceImport(ctx, m)
		if err != nil {
			return nil, err
		}
	}

	return &notes.ServiceNoteImportResponse{}, nil
}
